@@ -51,12 +51,7 @@ from cloudify.manager import update_execution_status, get_rest_client
 from cloudify.constants import LOGGING_CONFIG_FILE
 from cloudify.error_handling import serialize_known_exception
 
-try:
-    from cloudify.workflows import api
-    from cloudify.workflows import workflow_context
-except ImportError:
-    workflow_context = None
-    api = None
+from cloudify.workflows import api
 
 
 class TaskHandler(object):
@@ -149,6 +144,9 @@ class OperationHandler(TaskHandler):
     def _run_operation_func(self, ctx, kwargs):
         try:
             return self.func(*self.args, **kwargs)
+        except Exception as e:
+            err = _WrappedTaskError(e, traceback.format_exc())
+            raise err
         finally:
             if ctx.type == constants.NODE_INSTANCE:
                 ctx.instance.update()
@@ -157,15 +155,33 @@ class OperationHandler(TaskHandler):
                 ctx.target.instance.update()
 
 
-class WorkflowHandler(TaskHandler):
+class _WrappedTaskError(Exception):
+    """This wraps any exception coming out of a workflow/operation function.
 
+    We'll wrap the exception and the traceback, so that we can
+    preserve the original traceback. We want the original traceback
+    to be around, so that we can only print that, without adding
+    framework-level frames that come from dispatch.py itself.
+    """
+    def __init__(self, wrapped_exc, wrapped_tb, *args, **kwargs):
+        super(_WrappedTaskError, self).__init__(*args, **kwargs)
+        self.wrapped_exc = wrapped_exc
+        self.wrapped_tb = wrapped_tb
+
+
+class WorkflowHandler(TaskHandler):
     def __init__(self, *args, **kwargs):
-        if workflow_context is None or api is None:
+        if api is None:
             raise RuntimeError('Dispatcher not installed')
         super(WorkflowHandler, self).__init__(*args, **kwargs)
 
     @property
     def ctx_cls(self):
+        # import workflow_context in-function so that it doesn't need to
+        # be imported when handling an operation
+        # (only when handling a workflow)
+        from cloudify.workflows import workflow_context
+
         if getattr(self.func, 'workflow_system_wide', False):
             return workflow_context.CloudifySystemWideWorkflowContext
         return workflow_context.CloudifyWorkflowContext
@@ -279,6 +295,11 @@ class WorkflowHandler(TaskHandler):
             else:
                 self._workflow_succeeded()
             return result
+        except _WrappedTaskError as e:
+            self._workflow_failed(e.wrapped_exc, e.wrapped_tb)
+            # `raise e.wrapped_exc from None` - but syntax that won't break py2
+            e.wrapped_exc.__suppress_context__ = True
+            raise e.wrapped_exc
         except BaseException as e:
             self._workflow_failed(e, traceback.format_exc())
             raise
@@ -290,18 +311,19 @@ class WorkflowHandler(TaskHandler):
         with state.current_workflow_ctx.push(self.ctx, self.kwargs):
             try:
                 workflow_result = self._execute_workflow_function()
-                queue.put({'result': workflow_result})
-            except api.ExecutionCancelled:
-                queue.put({'result': api.EXECUTION_CANCELLED_RESULT})
-            except BaseException as workflow_ex:
+                queue.put(workflow_result)
+            except Exception as workflow_ex:
                 queue.put({'error': workflow_ex})
 
     def _handle_local_workflow(self):
         try:
             self._workflow_started()
             result = self._execute_workflow_function()
+            if 'error' in result:
+                wrapped_exc = result['error'].wrapped_exc
+                raise wrapped_exc
             self._workflow_succeeded()
-            return result
+            return result['result']
         except Exception as e:
             error = StringIO()
             traceback.print_exc(file=error)
@@ -309,15 +331,21 @@ class WorkflowHandler(TaskHandler):
             raise
 
     def _execute_workflow_function(self):
-        try:
-            self.ctx.internal.start_local_tasks_processing()
-            result = self.func(*self.args, **self.kwargs)
-            if not self.ctx.internal.graph_mode:
-                for workflow_task in self.ctx.internal.task_graph.tasks:
-                    workflow_task.async_result.get()
+        with self.ctx.internal.local_tasks_processor:
+            try:
+                workflow_output = self.func(*self.args, **self.kwargs)
+                result = {'result': workflow_output}
+                if not self.ctx.internal.graph_mode:
+                    for workflow_task in self.ctx.internal.task_graph.tasks:
+                        workflow_task.async_result.get()
+            except api.ExecutionCancelled:
+                result = {'result': api.EXECUTION_CANCELLED_RESULT}
+                return result
+            except BaseException as workflow_ex:
+                err = _WrappedTaskError(workflow_ex, traceback.format_exc())
+                result = {'error': err}
+                return result
             return result
-        finally:
-            self.ctx.internal.stop_local_tasks_processing()
 
     def _workflow_started(self):
         self._update_execution_status(Execution.STARTED)
@@ -348,7 +376,11 @@ class WorkflowHandler(TaskHandler):
                     self.ctx.workflow_id, exception),
                 args={'error': error_traceback},
             )
-            self._update_execution_status(Execution.FAILED, error_traceback)
+            if getattr(exception, 'hide_traceback', False):
+                tb = str(exception)
+            else:
+                tb = error_traceback
+            self._update_execution_status(Execution.FAILED, tb)
         except Exception:
             logger = logging.getLogger(__name__)
             logger.exception('Exception raised when attempting to update '
@@ -425,14 +457,18 @@ def main():
         _setup_logging()
         payload = handler.handle()
         payload_type = 'result'
+    except _WrappedTaskError as e:
+        payload_type = 'error'
+        payload = serialize_known_exception(e.wrapped_exc, e.wrapped_tb)
     except BaseException as e:
         payload_type = 'error'
         payload = serialize_known_exception(e)
 
+    if payload_type == 'error':
         logger = logging.getLogger(__name__)
         logger.error('Task {0}[{1}] raised:\n{2}'.format(
             handler.cloudify_context['task_name'],
-            handler.cloudify_context.get('task_id', '<no-id>'),
+            handler.cloudify_context.get('task_id', '<no-id>').strip(),
             payload.get('traceback')))
 
     with open(os.path.join(dispatch_dir, 'output.json'), 'w') as f:
