@@ -21,13 +21,17 @@ import datetime
 
 from cloudify import constants, manager
 from cloudify import event as _event
-from cloudify.utils import (get_execution_creator_username,
-                            get_execution_id,
-                            is_management_environment,
-                            ENV_AGENT_LOG_LEVEL,
-                            ENV_AGENT_LOG_DIR,
-                            ENV_AGENT_LOG_MAX_BYTES,
-                            ENV_AGENT_LOG_MAX_HISTORY)
+from cloudify.utils import (
+    get_execution_creator_username,
+    get_execution_id,
+    is_management_environment,
+    ENV_AGENT_LOG_LEVEL,
+    ENV_AGENT_LOG_DIR,
+    ENV_AGENT_LOG_MAX_BYTES,
+    ENV_AGENT_LOG_MAX_HISTORY,
+    get_manager_name,
+    get_daemon_name,
+)
 from cloudify._compat import text_type
 
 EVENT_CLASS = _event.Event
@@ -337,8 +341,12 @@ def stdout_log_out(log, **kwargs):
         sys.stdout.flush()
 
 
-def create_event_message_prefix(event):
-    event_obj = EVENT_CLASS(event, verbosity_level=EVENT_VERBOSITY_LEVEL)
+def create_event_message_prefix(event, with_worker_names=False):
+    event_obj = EVENT_CLASS(
+        event,
+        verbosity_level=EVENT_VERBOSITY_LEVEL,
+        with_worker_names=with_worker_names,
+    )
     if not event_obj.has_output:
         return None
     return text_type(event_obj)
@@ -371,9 +379,18 @@ def _publish_message(message, message_type, logger, skip_send=False):
     execution_id = get_execution_id()
     client = manager.get_rest_client()
     if message_type == 'log':
-        client.events.create(logs=[message], execution_id=execution_id)
+        logs = [message]
+        events = None
     else:
-        client.events.create(events=[message], execution_id=execution_id)
+        logs = None
+        events = [message]
+    client.events.create(
+        events=events,
+        logs=logs,
+        execution_id=execution_id,
+        agent_name=get_daemon_name(),
+        manager_name=get_manager_name(),
+    )
 
 
 def setup_logger_base(log_level, log_dir=None):
