@@ -1,4 +1,3 @@
-from __future__ import absolute_import
 ########
 # Copyright (c) 2015 GigaSpaces Technologies Ltd. All rights reserved
 #
@@ -14,4 +13,4 @@ from __future__ import absolute_import
 #    * See the License for the specific language governing permissions and
 #    * limitations under the License.
 
-from .local_workflow_decorator import WorkflowTestDecorator as workflow_test     # NOQA
+from cloudify.test_utils.local_workflow_decorator import WorkflowTestDecorator as workflow_test     # NOQA
