@@ -26,12 +26,12 @@ from __future__ import absolute_import
 import functools
 import copy
 import json
-import uuid
 import threading
 
-from proxy_tools import proxy
-
 from dsl_parser import functions as dsl_functions
+from dsl_parser.utils import parse_simple_type_value
+
+
 from cloudify import context
 from cloudify._compat import queue
 from cloudify.manager import (get_bootstrap_context,
@@ -55,7 +55,6 @@ from cloudify import utils, logs
 from cloudify.state import current_workflow_ctx
 from cloudify.workflows import events
 from cloudify.workflows.tasks_graph import TaskDependencyGraph
-from cloudify.workflows.amqp_dispatcher import TaskDispatcher
 from cloudify.logs import (CloudifyWorkflowLoggingHandler,
                            CloudifyWorkflowNodeLoggingHandler,
                            SystemWideWorkflowLoggingHandler,
@@ -94,9 +93,27 @@ class CloudifyWorkflowRelationshipInstance(object):
         self._relationship = node_instance.node.get_relationship(
             relationship_instance['target_name'])
 
+    def __repr__(self):
+        return '<{0} {1}->{2} ({3})>'.format(
+            self.__class__.__name__,
+            self.source_id,
+            self.target_id,
+            self.type,
+        )
+
+    @property
+    def type(self):
+        """The relationship type"""
+        return self._relationship_instance.get('type')
+
+    @property
+    def source_id(self):
+        """The relationship source node-instance id"""
+        return self.node_instance.id
+
     @property
     def target_id(self):
-        """The relationship target node id"""
+        """The relationship target node-instance id"""
         return self._relationship_instance.get('target_id')
 
     @property
@@ -235,6 +252,27 @@ class CloudifyWorkflowNodeInstance(object):
 
         self._logger = None
 
+    def __eq__(self, other):
+        """Compare node instances
+
+        A node instance is always going to be equal with itself, even if it
+        was fetched multiple times, and it was mutated by the user.
+        This allows storing node instances in sets, and as dict keys.
+        """
+        if not isinstance(other, self.__class__):
+            return False
+        return self.id == other.id
+
+    def __hash__(self):
+        return hash(self.id)
+
+    def __repr__(self):
+        return '<{0} {1} ({2})>'.format(
+            self.__class__.__name__,
+            self.id,
+            self.state,
+        )
+
     def set_state(self, state):
         """Set the node-instance state
 
@@ -303,6 +341,18 @@ class CloudifyWorkflowNodeInstance(object):
         """The node instance state"""
         return self._node_instance.state
 
+    @property
+    def version(self):
+        """The node instance version
+
+        Node-instance storage uses an optimistic concurrency control approach:
+        when updating a node-instance, also include the version that the
+        client thinks is current (ie. this value).
+        If the server has a more recent version, it will return an error,
+        allowing the client to fetch the more recent version and retry.
+        """
+        return self._node_instance.version
+
     @property
     def node_id(self):
         """The node id (this instance is an instance of that node)"""
@@ -327,6 +377,18 @@ class CloudifyWorkflowNodeInstance(object):
     def scaling_groups(self):
         return self._node_instance.get('scaling_groups', [])
 
+    @property
+    def runtime_properties(self):
+        """The node instance runtime properties
+
+        Note that in workflow code, it is common for runtime-properties
+        to be outdated, if a prior operation changed them.
+        Before using this value, consider if it is up to date. You can
+        use the refresh_node_instances method to bring all node-instance
+        properties up to date.
+        """
+        return self._node_instance.runtime_properties
+
     @property
     def system_properties(self):
         return self._node_instance.get('system_properties', {})
@@ -383,6 +445,12 @@ class CloudifyWorkflowNode(object):
             for relationship in node.relationships)
         self._node_instances = {}
 
+    def __repr__(self):
+        return '<{0} {1})>'.format(
+            self.__class__.__name__,
+            self.id,
+        )
+
     @property
     def id(self):
         """The node id"""
@@ -444,6 +512,12 @@ class CloudifyWorkflowNode(object):
         """The node instances"""
         return iter(self._node_instances.values())
 
+    def has_operation(self, operation_interface):
+        try:
+            return bool(self.operations[operation_interface]['operation'])
+        except KeyError:
+            return False
+
     def get_relationship(self, target_id):
         """Get a node relationship by its target id"""
         return self._relationships.get(target_id)
@@ -676,6 +750,9 @@ class _WorkflowContextBase(object):
         final_kwargs = self._merge_dicts(merged_from=kwargs,
                                          merged_into=operation_properties,
                                          allow_override=allow_kwargs_override)
+        operation_properties_types = op_struct.get('inputs_types')
+        if operation_properties_types:
+            _validate_types(operation_properties_types, final_kwargs)
 
         return self.execute_task(
             task_name,
@@ -756,7 +833,7 @@ class _WorkflowContextBase(object):
         # Should deepcopy cause problems here, remove it, but please make
         # sure that WORKFLOWS_WORKER_PAYLOAD is not global in manager repo
         kwargs = copy.deepcopy(kwargs) or {}
-        task_id = str(uuid.uuid4())
+        task_id = utils.uuid4()
         cloudify_context = self._build_cloudify_context(
             task_id,
             task_name,
@@ -979,6 +1056,9 @@ class _WorkflowContextBase(object):
     def delete_node(self, deployment_id, node_id):
         self.internal.handler.delete_node(deployment_id, node_id)
 
+    def delete_node_instance(self, node_id):
+        self.internal.handler.delete_node_instance(node_id)
+
     def create_nodes(self, deployment_id, nodes):
         self.internal.handler.create_nodes(deployment_id, nodes)
 
@@ -1128,53 +1208,11 @@ class CloudifyWorkflowContext(_WorkflowContextBase):
 
 
 class CloudifySystemWideWorkflowContext(_WorkflowContextBase):
-
     def __init__(self, ctx):
         super(CloudifySystemWideWorkflowContext, self).__init__(
             ctx,
             SystemWideWfRemoteContextHandler
         )
-        self._dep_contexts = None
-
-    class _ManagedCloudifyWorkflowContext(CloudifyWorkflowContext):
-        def __enter__(self):
-            self.internal.start_local_tasks_processing()
-
-        def __exit__(self, *args, **kwargs):
-            self.internal.stop_local_tasks_processing()
-
-    @property
-    def deployments_contexts(self):
-        if self.local:
-            raise RuntimeError(
-                'deployment_contexts do not exist in local workflows')
-        if self._dep_contexts is None:
-            self._dep_contexts = {}
-
-            deployments_list = \
-                self.internal.handler.rest_client.deployments.list(
-                    _include=['id', 'blueprint_id'],
-                    _get_all_results=True
-                )
-            for dep in deployments_list:
-                # Failure to deepcopy will cause snapshot restore context hack
-                # to be reset just before it's needed.
-                dep_ctx = copy.deepcopy(self._context)
-                dep_ctx['tenant']['name'] = self.tenant_name
-                dep_ctx['deployment_id'] = dep.id
-                dep_ctx['blueprint_id'] = dep.blueprint_id
-
-                def lazily_loaded_ctx(dep_ctx):
-                    def lazy_ctx():
-                        if not hasattr(lazy_ctx, '_cached_ctx'):
-                            lazy_ctx._cached_ctx = \
-                                self._ManagedCloudifyWorkflowContext(dep_ctx)
-                        return lazy_ctx._cached_ctx
-
-                    return proxy(lazy_ctx)
-
-                self._dep_contexts[dep.id] = lazily_loaded_ctx(dep_ctx)
-        return self._dep_contexts
 
 
 class CloudifyWorkflowContextInternal(object):
@@ -1248,12 +1286,6 @@ class CloudifyWorkflowContextInternal(object):
                                          args=args,
                                          additional_context=additional_context)
 
-    def start_local_tasks_processing(self):
-        self.local_tasks_processor.start()
-
-    def stop_local_tasks_processing(self):
-        self.local_tasks_processor.stop()
-
     def add_local_task(self, task):
         self.local_tasks_processor.add_task(task)
 
@@ -1276,6 +1308,12 @@ class LocalTasksProcessing(object):
             self._local_task_processing_pool.append(thread)
         self.stopped = False
 
+    def __enter__(self):
+        self.start()
+
+    def __exit__(self, exc_type, exc_val, tb):
+        self.stop()
+
     def start(self):
         for thread in self._local_task_processing_pool:
             thread.start()
@@ -1424,6 +1462,9 @@ class CloudifyWorkflowContextHandler(object):
     def delete_node(self, deployment_id, node_id):
         raise NotImplementedError('Implemented by subclasses')
 
+    def delete_node_instance(self, instance_id):
+        raise NotImplementedError('Implemented by subclasses')
+
     def create_nodes(self, deployment_id, nodes):
         raise NotImplementedError('Implemented by subclasses')
 
@@ -1459,7 +1500,7 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
     def __init__(self, *args, **kwargs):
         super(RemoteContextHandler, self).__init__(*args, **kwargs)
         self._rest_client = None
-        self._dispatcher = TaskDispatcher(self.workflow_ctx)
+        self._dispatcher = None
         self._plugins_cache = {}
         self._bootstrap_context = None
 
@@ -1470,7 +1511,7 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
         return self._rest_client
 
     def cleanup(self, finished):
-        if finished:
+        if finished and self._dispatcher is not None:
             self._dispatcher.cleanup()
 
     @property
@@ -1482,10 +1523,21 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
     def get_send_task_event_func(self, task):
         return events.send_task_event_func_remote
 
+    def _prepare_dispatcher(self):
+        # only import TaskDispatcher if we actually want to dispatch tasks -
+        # some workflows (eg. upload_blueprint, or create-dep-env) don't
+        # need to do so, so they don't need to pay the price of importing pika
+        from cloudify.workflows.amqp_dispatcher import TaskDispatcher
+        self._dispatcher = TaskDispatcher(self.workflow_ctx)
+
     def send_task(self, *args, **kwargs):
+        if self._dispatcher is None:
+            self._prepare_dispatcher()
         return self._dispatcher.send_task(*args, **kwargs)
 
     def wait_for_result(self, *args, **kwargs):
+        if self._dispatcher is None:
+            self._prepare_dispatcher()
         return self._dispatcher.wait_for_result(*args, **kwargs)
 
     @property
@@ -1539,7 +1591,10 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
             exception_causes = getattr(exception, 'causes', None)
         self.rest_client.operations.update(
             operation_id, state=state, result=result,
-            exception=exception_text, exception_causes=exception_causes)
+            exception=exception_text, exception_causes=exception_causes,
+            agent_name=utils.get_daemon_name(),
+            manager_name=utils.get_manager_name(),
+        )
 
     def _update_operation_inputs(self, *args, **kwargs):
         self.rest_client.operations._update_operation_inputs(*args, **kwargs)
@@ -1638,6 +1693,9 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
     def delete_node(self, deployment_id, node_id):
         self.rest_client.nodes.delete(deployment_id, node_id)
 
+    def delete_node_instance(self, instance_id):
+        self.rest_client.node_instances.delete(instance_id)
+
     def create_nodes(self, deployment_id, nodes):
         self.rest_client.nodes.create_many(deployment_id, nodes)
 
@@ -1895,6 +1953,9 @@ class LocalCloudifyWorkflowContextHandler(CloudifyWorkflowContextHandler):
     def delete_node(self, deployment_id, node_id):
         self.storage.delete_node(deployment_id, node_id)
 
+    def delete_node_instance(self, instance_id):
+        raise NotImplementedError('Not implemented yet')
+
     def create_nodes(self, deployment_id, nodes):
         self.storage.create_nodes(deployment_id, nodes)
 
@@ -2035,7 +2096,7 @@ class WorkflowDeploymentContext(context.DeploymentContext):
 
     @property
     def resource_tags(self):
-        """Resource tags associated wth this deployment."""
+        """Resource tags associated with this deployment."""
         if self._resource_tags is None and self.workflow_ctx.internal:
             raw_tags = self._context.get('deployment_resource_tags')
             if raw_tags:
@@ -2061,3 +2122,20 @@ def task_config(fn=None, **arguments):
         def partial_wrapper(func):
             return task_config(func, **arguments)
         return partial_wrapper
+
+
+def _validate_types(schema, arguments):
+    if not isinstance(schema, dict):
+        return True
+    for input_name, type_name in schema.items():
+        if input_name not in arguments:
+            continue
+        if dsl_functions.get_function(arguments[input_name]):
+            continue
+        _, valid = parse_simple_type_value(arguments[input_name], type_name)
+        if not valid:
+            raise RuntimeError(
+                "Value {0} of '{1}' does not match the definition: {2}"
+                .format(arguments[input_name], input_name, type_name)
+            )
+    return True
