@@ -1,18 +1,3 @@
-########
-# Copyright (c) 2018-2019 Cloudify Platform Ltd. All rights reserved
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    * See the License for the specific language governing permissions and
-#    * limitations under the License.
-
 import json
 import logging
 import numbers
@@ -42,6 +27,7 @@ from cloudify_rest_client.tenants import TenantsClient
 from cloudify_rest_client.evaluate import EvaluateClient
 from cloudify_rest_client.summary import SummariesClient
 from cloudify_rest_client.snapshots import SnapshotsClient
+from cloudify_rest_client.log_bundles import LogBundlesClient
 from cloudify_rest_client.cluster import ClusterStatusClient
 from cloudify_rest_client.blueprints import BlueprintsClient
 from cloudify_rest_client.executions import (
@@ -58,7 +44,7 @@ from cloudify_rest_client.permissions import PermissionsClient
 from cloudify_rest_client.maintenance import MaintenanceModeClient
 from cloudify_rest_client.plugins_update import PluginsUpdateClient
 from cloudify_rest_client.node_instances import NodeInstancesClient
-from cloudify_rest_client.tokens import TokensClient, UserTokensClient
+from cloudify_rest_client.tokens import TokensClient
 from cloudify_rest_client.deployment_updates import DeploymentUpdatesClient
 from cloudify_rest_client.operations import OperationsClient, TasksGraphClient
 from cloudify_rest_client.deployment_modifications import (
@@ -138,7 +124,6 @@ class HTTPClient(object):
 
     def has_auth_header(self):
         auth_headers = [constants.CLOUDIFY_AUTHENTICATION_HEADER,
-                        constants.CLOUDIFY_API_AUTH_TOKEN_HEADER,
                         constants.CLOUDIFY_EXECUTION_TOKEN_HEADER,
                         constants.CLOUDIFY_TOKEN_AUTHENTICATION_HEADER]
         return any(header in self.headers for header in auth_headers)
@@ -474,6 +459,7 @@ class CloudifyClient(object):
         self.idp = IdentityProviderClient(self._client)
         self.permissions = PermissionsClient(self._client)
         self.snapshots = SnapshotsClient(self._client)
+        self.log_bundles = LogBundlesClient(self._client)
         self.deployments = DeploymentsClient(self._client)
         self.deployment_groups = DeploymentGroupsClient(self._client)
         self.executions = ExecutionsClient(self._client)
@@ -498,7 +484,6 @@ class CloudifyClient(object):
         self.secrets = SecretsClient(self._client)
         self.agents = AgentsClient(self._client)
         self.summary = SummariesClient(self._client)
-        self.user_tokens = UserTokensClient(self._client)
         self.operations = OperationsClient(self._client)
         self.tasks_graphs = TasksGraphClient(self._client)
         self.license = LicenseClient(self._client)
