@@ -20,7 +20,7 @@ from dsl_parser import (constants,
                         exceptions,
                         utils)
 from dsl_parser._compat import text_type
-from dsl_parser.elements import (data_types,
+from dsl_parser.elements import (inputs,
                                  version as _version)
 from dsl_parser.framework.elements import (DictElement,
                                            Element,
@@ -62,7 +62,7 @@ class OperationExecutor(Element):
                             ','.join(valid_executors)))
 
 
-class NodeTypeOperationInputs(data_types.Schema):
+class NodeTypeOperationInputs(inputs.Inputs):
     add_namespace_to_schema_elements = False
 
 
@@ -250,15 +250,19 @@ def _is_remote_script_resource(resource_path, remote_resources_namespaces):
             remote_resources_namespaces)
 
 
-def workflow_operation(plugin_name,
-                       workflow_mapping,
-                       workflow_parameters,
-                       is_workflow_cascading):
+def workflow_operation(
+    plugin_name,
+    workflow_mapping,
+    workflow_parameters,
+    is_workflow_cascading,
+    workflow_availability,
+):
     return {
         'plugin': plugin_name,
         'operation': workflow_mapping,
         'parameters': workflow_parameters,
-        'is_cascading': is_workflow_cascading
+        'is_cascading': is_workflow_cascading,
+        'availability_rules': workflow_availability,
     }
 
 
@@ -271,12 +275,19 @@ def process_operation(
         resource_bases,
         remote_resources_namespaces,
         is_workflows=False,
-        is_workflow_cascading=False):
+        is_workflow_cascading=False,
+        workflow_availability=None,
+):
     payload_field_name = 'parameters' if is_workflows else 'inputs'
     mapping_field_name = 'mapping' if is_workflows else 'implementation'
     operation_mapping = operation_content[mapping_field_name]
     operation_payload = operation_content[payload_field_name]
 
+    operation_payload_types = \
+        {k: v['type']
+         for k, v in operation_payload.items()
+         if isinstance(v, dict) and 'type' in v}
+
     # only for node operations
     operation_executor = operation_content.get('executor', None)
     operation_max_retries = operation_content.get('max_retries', None)
@@ -346,7 +357,9 @@ def process_operation(
                 plugin_name=script_plugin,
                 workflow_mapping=operation_mapping,
                 workflow_parameters=operation_payload,
-                is_workflow_cascading=is_workflow_cascading)
+                is_workflow_cascading=is_workflow_cascading,
+                workflow_availability=workflow_availability,
+            )
         else:
             if not operation_executor:
                 operation_executor = plugins[script_plugin]['executor']
@@ -359,7 +372,9 @@ def process_operation(
                 max_retries=operation_max_retries,
                 retry_interval=operation_retry_interval,
                 timeout=operation_timeout,
-                timeout_recoverable=operation_timeout_recoverable)
+                timeout_recoverable=operation_timeout_recoverable,
+                operation_inputs_types=operation_payload_types,
+            )
 
     if candidate_plugins:
         if len(candidate_plugins) > 1:
@@ -373,7 +388,9 @@ def process_operation(
                 plugin_name=plugin_name,
                 workflow_mapping=mapping,
                 workflow_parameters=operation_payload,
-                is_workflow_cascading=is_workflow_cascading)
+                is_workflow_cascading=is_workflow_cascading,
+                workflow_availability=workflow_availability,
+            )
         else:
             if not operation_executor:
                 operation_executor = plugins[plugin_name]['executor']
@@ -386,7 +403,9 @@ def process_operation(
                 max_retries=operation_max_retries,
                 retry_interval=operation_retry_interval,
                 timeout=operation_timeout,
-                timeout_recoverable=operation_timeout_recoverable)
+                timeout_recoverable=operation_timeout_recoverable,
+                operation_inputs_types=operation_payload_types,
+            )
 
     else:
         base_error_message = (
