@@ -15,6 +15,7 @@
 
 import abc
 import collections
+
 import pkg_resources
 import json
 
@@ -28,6 +29,10 @@ from dsl_parser.constants import (OUTPUTS,
                                   INTER_DEPLOYMENT_FUNCTIONS,
                                   EVAL_FUNCS_PATH_PREFIX_KEY,
                                   EVAL_FUNCS_PATH_DEFAULT_PREFIX)
+from dsl_parser.utils import (TEMPLATE_FUNCTIONS,
+                              get_function,
+                              parse_simple_type_value)
+
 
 SELF = 'SELF'
 SOURCE = 'SOURCE'
@@ -39,8 +44,6 @@ AVAILABLE_NODE_TARGETS = [SELF, SOURCE, TARGET]
 # Runtime can be evaluated only at runtime.
 STATIC_FUNC, HYBRID_FUNC, RUNTIME_FUNC = [0, 1, 2]
 
-TEMPLATE_FUNCTIONS = {}
-
 
 class RecursionLimit(object):
     def __init__(self, limit, args_to_str_func=None):
@@ -120,14 +123,6 @@ def _register_entry_point_functions():
         register(fn=entry_point.load(), name=entry_point.name)
 
 
-def is_function(value):
-    """Does the value represent a template function call?"""
-    # functions use the syntax {function_name: args}, so let's look for
-    # dicts of length 1 where the only key was registered as a function
-    return isinstance(value, dict) and len(value) == 1 \
-        and list(value.keys())[0] in TEMPLATE_FUNCTIONS
-
-
 def _convert_attribute_list_to_python_syntax_string(attr_list):
     """This converts the given attribute list to Python syntax. For example,
     calling this function with ['obj1', 'attr1', 7, 'attr2'] will output
@@ -151,7 +146,8 @@ _register_entry_point_functions()
 
 
 def _contains_legal_nested_attribute_path_items(args):
-    return all(is_function(x) or isinstance(x, (text_type, int)) for x in args)
+    return all(get_function(x) or isinstance(x, (text_type, int))
+               for x in args)
 
 
 def _is_legal_nested_attribute_path(args):
@@ -164,7 +160,8 @@ class Function(ABC):
     name = 'function'
     func_eval_type = None
 
-    def __init__(self, args, scope=None, context=None, path=None, raw=None):
+    def __init__(self, args, scope=None, context=None, path=None, raw=None,
+                 return_type=None):
         """Initializes the instance.
 
         :param args: function argument. For Static functions, these arguments
@@ -182,6 +179,7 @@ class Function(ABC):
         self.context = context
         self.path = path
         self.raw = raw
+        self.return_type = return_type
         self.parse_args(args)
 
     @abc.abstractmethod
@@ -214,7 +212,7 @@ class GetInput(Function):
                 and _contains_legal_nested_attribute_path_items(args_list)
 
         if not isinstance(args, text_type) \
-                and not is_function(args) \
+                and not get_function(args) \
                 and not _is_valid_args_list(args):
             raise ValueError(
                 "Illegal argument(s) passed to {0} function. Expected either "
@@ -226,7 +224,7 @@ class GetInput(Function):
     def validate(self, plan):
         input_value = self.input_value[0] \
             if isinstance(self.input_value, list) else self.input_value
-        if is_function(input_value):
+        if get_function(input_value):
             return
         if input_value not in plan.inputs:
             raise exceptions.UnknownInputError(
@@ -234,20 +232,31 @@ class GetInput(Function):
                 "unknown input '{0}'.".format(input_value))
 
     def evaluate(self, handler):
+        return_value = None
         if isinstance(self.input_value, list):
-            if any(is_function(x) for x in self.input_value):
+            if any(get_function(x) for x in self.input_value):
                 raise exceptions.FunctionEvaluationError(
                     '{0}: found an unresolved argument: {1}'
                     .format(self.name, self.input_value))
 
-            return self._get_input_attribute(
+            return_value = self._get_input_attribute(
                 handler.get_input(self.input_value[0]))
 
-        if is_function(self.input_value):
+        if get_function(self.input_value):
             raise exceptions.FunctionEvaluationError(
                 '{0}: found an unresolved argument: {1}'
                 .format(self.name, self.input_value))
-        return handler.get_input(self.input_value)
+        return_value = return_value or handler.get_input(self.input_value)
+
+        if self.return_type:
+            _, ok = parse_simple_type_value(return_value, self.return_type)
+            if not ok:
+                raise exceptions.InputEvaluationError(
+                    "Value '{0}' of input '{1}' does not match data type "
+                    "declared for '{2}': '{3}'."
+                    .format(return_value, self.input_value, self.path,
+                            self.return_type))
+        return return_value
 
     def _get_input_attribute(self, root):
         value = root
@@ -295,6 +304,7 @@ class GetInput(Function):
 class GetSys(Function):
     VALID_PROPERTIES = [('tenant', 'name'),
                         ('deployment', 'id'),
+                        ('deployment', 'name'),
                         ('deployment', 'blueprint'),
                         ('deployment', 'owner')]
 
@@ -317,14 +327,14 @@ class GetSys(Function):
 
     def validate(self, plan):
         known_entities = set(p[0] for p in self.VALID_PROPERTIES)
-        if is_function(self.entity):
+        if get_function(self.entity):
             return
         if not isinstance(self.entity, text_type) \
                 or self.entity not in known_entities:
             raise exceptions.UnknownSysEntityError(
                 "{0} function unable to determine entity: {1}"
                 .format(self.name, self.entity))
-        if is_function(self.property):
+        if get_function(self.property):
             return
         if not isinstance(self.property, text_type) \
                 or (self.entity, self.property) not in self.VALID_PROPERTIES:
@@ -336,6 +346,31 @@ class GetSys(Function):
         return handler.get_sys(self.entity, self.property)
 
 
+@register(name='get_consumers', func_eval_type=RUNTIME_FUNC)
+class GetConsumers(Function):
+    VALID_PROPERTIES = ['ids', 'names', 'count']
+
+    def __init__(self, args, **kwargs):
+        self.property = None
+        super(GetConsumers, self).__init__(args, **kwargs)
+
+    def parse_args(self, args):
+        if isinstance(args, text_type) and args in self.VALID_PROPERTIES:
+            self.property = args
+        else:
+            raise ValueError(
+                "Illegal argument passed to {name} function. Expected a "
+                "property string (one of: {props}), but got {args}".format(
+                    name=self.name, props=', '.join(self.VALID_PROPERTIES),
+                    args=args))
+
+    def validate(self, plan):
+        pass
+
+    def evaluate(self, handler):
+        return handler.get_consumers(self.property)
+
+
 @register(name='get_property', func_eval_type=HYBRID_FUNC)
 class GetProperty(Function):
 
@@ -357,7 +392,7 @@ class GetProperty(Function):
     def validate(self, plan):
         # Try to evaluate only when the arguments don't contain any functions
         args = [self.node_name] + self.property_path
-        if any(is_function(x) for x in args):
+        if any(get_function(x) for x in args):
             return
         handler = _PlanEvaluationHandler(plan, runtime_only_evaluation=False)
         self.evaluate(handler)
@@ -389,22 +424,26 @@ class GetProperty(Function):
             # that the EvaluationHandler can use the new node as the context,
             # in case there's another get_property call in the result
             self.context = node
-        self._get_property_value(node)
+        self._get_property_value(node, handler)
         return node
 
-    def _get_property_value(self, node_template):
+    def _get_property_value(self, node_template, handler):
         return _get_property_value(node_template['name'],
                                    node_template.get('properties', {}),
                                    self.property_path,
-                                   self.path)
+                                   handler,
+                                   self.context,
+                                   self.path,
+                                   func_name='get_property')
 
     def evaluate(self, handler):
-        if is_function(self.node_name) or \
-                any(is_function(x) for x in self.property_path):
+        if get_function(self.node_name) or \
+                any(get_function(x) for x in self.property_path):
             raise exceptions.FunctionEvaluationError(
                 '{0}: found an unresolved argument in path: {1}, {2}'
                 .format(self.name, self.node_name, self.property_path))
-        return self._get_property_value(self.get_node_template(handler))
+        return self._get_property_value(self.get_node_template(handler),
+                                        handler)
 
 
 @register(name='get_attribute', func_eval_type=RUNTIME_FUNC)
@@ -431,14 +470,13 @@ class GetAttribute(Function):
 
     def evaluate(self, handler):
         if self.node_name in [SELF, SOURCE, TARGET]:
-            node_instance_id = self.context.get(self.node_name.lower())
+            node_instance_id = self._resolve_available_node_targets(handler)
             _validate_ref(node_instance_id, self.node_name, self.name,
                           self.path, self.attribute_path)
             node_instance = handler.get_node_instance(node_instance_id)
         else:
             try:
                 node_instance = self._resolve_node_instance_by_name(handler)
-                node_instance_id = node_instance['id']
             except exceptions.FunctionEvaluationError as e:
                 # Only in outputs scope we allow to continue when an error
                 # occurred
@@ -452,11 +490,23 @@ class GetAttribute(Function):
             self.context = self.context.copy()
             self.context['self'] = node_instance['id']
         value = _get_attribute_from_node_instance(
-            node_instance, node, self.attribute_path, self.path)
+            node_instance, node, self.attribute_path, self.path,
+            handler, self.context, 'get_attribute')
         return value
 
-    def _resolve_node_instance_by_name(self, handler):
-        node_id = self.node_name
+    def _resolve_available_node_targets(self, handler):
+        node_instance_id = self.context.get(self.node_name.lower())
+        if node_instance_id:
+            return node_instance_id
+
+        if self.node_name == SELF:
+            node_instance = self._resolve_node_instance_by_name(
+                handler, node_id=self.context.get('id'))
+            if node_instance:
+                return node_instance.get('id')
+
+    def _resolve_node_instance_by_name(self, handler, node_id=None):
+        node_id = node_id or self.node_name
         node_instances = handler.get_node_instances(node_id)
 
         if len(node_instances) == 0:
@@ -595,12 +645,17 @@ def _validate_ref(ref, ref_name, name, path, attribute_path):
                                    attribute_path))
 
 
-def _get_attribute_from_node_instance(ni, node, attribute_path, path):
+def _get_attribute_from_node_instance(ni, node, attribute_path, path,
+                                      handler, context, fn_name):
     value = _get_property_value(ni['node_id'],
                                 ni.get('runtime_properties'),
                                 attribute_path,
+                                handler,
+                                context,
                                 path,
-                                raise_if_not_found=False)
+                                raise_if_not_found=False,
+                                func_name=fn_name)
+
     if value is None:
         # attribute not found in instance runtime properties
         # special case for { get_attributes_list: [
@@ -614,8 +669,11 @@ def _get_attribute_from_node_instance(ni, node, attribute_path, path):
         value = _get_property_value(node['id'],
                                     node.get('properties', {}),
                                     attribute_path,
+                                    handler,
+                                    context,
                                     path,
-                                    raise_if_not_found=False)
+                                    raise_if_not_found=False,
+                                    func_name=fn_name)
     return value
 
 
@@ -623,7 +681,7 @@ def _validate_no_functions_in_args(node_name, attribute_path, path, name,
                                    scope, plan):
     # If any of the arguments are functions don't validate
     args = [node_name] + attribute_path
-    if any(is_function(x) for x in args):
+    if any(get_function(x) for x in args):
         return
 
     if node_name in [SELF, SOURCE, TARGET]:
@@ -682,7 +740,9 @@ class GetAttributesList(Function):
 
         for ni in node_instances:
             results.append(_get_attribute_from_node_instance(
-                ni, node, self.attribute_path, self.path))
+                ni, node, self.attribute_path, self.path,
+                handler, self.context, 'get_attributes_list'
+            ))
 
         return results
 
@@ -749,7 +809,9 @@ class GetAttributesDict(Function):
             for attribute_path in self.attribute_paths:
                 identifier = self._convert_to_identifier(attribute_path)
                 ni_result[identifier] = _get_attribute_from_node_instance(
-                    ni, node, attribute_path, self.path)
+                    ni, node, attribute_path, self.path,
+                    handler, self.context, 'get_attributes_dict'
+                )
             results[ni['id']] = ni_result
 
         return results
@@ -771,7 +833,7 @@ class GetSecret(Function):
             not (
                 (isinstance(args, list) and len(args) > 1)
                 or isinstance(args, text_type)
-                or is_function(args)
+                or get_function(args)
             )
         ):
             raise exceptions.FunctionValidationError(
@@ -922,7 +984,7 @@ class GetCapability(InterDeploymentDependencyCreatingFunction):
                 .format(','.join('{0}'.format(a) for a in args))
             )
         for arg_index, arg in enumerate(args):
-            if not isinstance(arg, (text_type, int)) and not is_function(arg):
+            if not isinstance(arg, (text_type, int)) and not get_function(arg):
                 raise ValueError(
                     "`get_capability` function arguments can't be complex "
                     "values; only strings/ints/functions are accepted. "
@@ -941,7 +1003,7 @@ class GetCapability(InterDeploymentDependencyCreatingFunction):
     @property
     def target_deployment(self):
         first_arg = self.capability_path[0]
-        target_deployment = None if is_function(first_arg) else first_arg
+        target_deployment = None if get_function(first_arg) else first_arg
         return target_deployment, first_arg
 
 
@@ -967,7 +1029,7 @@ class GetGroupCapability(Function):
         for arg_index, arg in enumerate(args):
             if arg_index == 1 and isinstance(arg, list):
                 continue
-            if not isinstance(arg, (text_type, int)) and not is_function(arg):
+            if not isinstance(arg, (text_type, int)) and not get_function(arg):
                 raise ValueError(
                     "`get_group_capability` function arguments can't be "
                     "complex values; only strings/ints/functions are "
@@ -992,7 +1054,7 @@ class GetLabel(Function):
 
     def parse_args(self, args):
         if isinstance(args, list) and len(args) == 2:
-            if not (isinstance(args[0], text_type) or is_function(args[0])):
+            if not (isinstance(args[0], text_type) or get_function(args[0])):
                 raise exceptions.FunctionValidationError(
                     '`get_label`',
                     "the <label-key> should be a string or a dict "
@@ -1010,7 +1072,7 @@ class GetLabel(Function):
             self.values_list_index = int(args[1])
         elif isinstance(args, text_type):
             self.label_key = args
-        elif is_function(args):
+        elif get_function(args):
             self.label_key = args
         else:
             raise exceptions.FunctionValidationError(
@@ -1057,7 +1119,7 @@ class GetEnvironmentCapability(Function):
                     args), args)
             )
         for arg_index, arg in enumerate(args):
-            if not isinstance(arg, (text_type, int)) and not is_function(arg):
+            if not isinstance(arg, (text_type, int)) and not get_function(arg):
                 raise ValueError(
                     "`get_environment_capability` function arguments"
                     " can't be complex values; only strings/ints/functions"
@@ -1080,7 +1142,7 @@ class ValidateArgumentMixin(object):
         exception_cls = exceptions.FunctionValidationError if validation_only \
             else exceptions.FunctionEvaluationError
         value = getattr(self, argument_name)
-        if is_function(value):
+        if get_function(value):
             if validation_only:
                 return
             raise exception_cls(
@@ -1237,8 +1299,11 @@ class StringUpper(Function, ValidateArgumentMixin):
 def _get_property_value(node_name,
                         properties,
                         property_path,
+                        handler,
+                        context,
                         context_path='',
-                        raise_if_not_found=True):
+                        raise_if_not_found=True,
+                        func_name=None):
     """Extracts a property's value according to the provided property path
 
     :param node_name: Node name the property belongs to (for logging).
@@ -1246,66 +1311,81 @@ def _get_property_value(node_name,
     :param property_path: Property path as list.
     :param context_path: Context path (for logging).
     :param raise_if_not_found: Whether to raise an error if property not found.
+    "param func_name: Originating function name (for raising error).
     :return: Property value.
     """
     def str_list(li):
         return [str(item) for item in li]
 
+    def evaluate_property_value(v):
+        if not handler.runtime_only_evaluation:
+            return v
+        return handler(v, None, context, context_path)
+
     value = properties
     for p in property_path:
-        if is_function(value):
+        if get_function(value):
             value = [value, p]
         elif isinstance(value, dict):
             if p not in value:
                 if raise_if_not_found:
-                    raise KeyError(
+                    raise exceptions.FunctionEvaluationError(
+                        func_name,
                         "Node template property '{0}.properties.{1}' "
-                        "referenced from '{2}' doesn't exist.".format(
-                            node_name, '.'.join(str_list(property_path)),
-                            context_path))
+                        "referenced from '{2}' doesn't exist."
+                        .format(node_name, '.'.join(str_list(property_path)),
+                                context_path)
+                    )
                 return None
             else:
-                value = value[p]
+                value = evaluate_property_value(value[p])
         elif isinstance(value, list):
             try:
-                value = value[p]
+                value = evaluate_property_value(value[p])
             except TypeError:
-                raise TypeError(
+                raise exceptions.FunctionEvaluationError(
+                    func_name,
                     "Node template property '{0}.properties.{1}' "
                     "referenced from '{2}' is expected {3} to be an int "
-                    "but it is a {4}.".format(
-                        node_name, '.'.join(str_list(property_path)),
-                        context_path,
-                        p, type(p).__name__))
+                    "but it is a {4}."
+                    .format(node_name, '.'.join(str_list(property_path)),
+                            context_path, p, type(p).__name__)
+                )
             except IndexError:
                 if raise_if_not_found:
-                    raise IndexError(
+                    raise exceptions.FunctionEvaluationError(
+                        func_name,
                         "Node template property '{0}.properties.{1}' "
                         "referenced from '{2}' index is out of range. Got {3}"
-                        " but list size is {4}.".format(
-                            node_name, '.'.join(str_list(property_path)),
-                            context_path, p, len(value)))
+                        " but list size is {4}."
+                        .format(node_name, '.'.join(str_list(property_path)),
+                                context_path, p, len(value))
+                    )
                 return None
         else:
             if raise_if_not_found:
-                raise KeyError(
+                raise exceptions.FunctionEvaluationError(
+                    func_name,
                     "Node template property '{0}.properties.{1}' "
-                    "referenced from '{2}' doesn't exist.".format(
-                        node_name, '.'.join(str_list(property_path)),
-                        context_path))
+                    "referenced from '{2}' doesn't exist."
+                    .format(node_name, '.'.join(str_list(property_path)),
+                            context_path)
+                )
             return None
 
     return value
 
 
 def parse(raw_function, scope=None, context=None, path=None):
-    if is_function(raw_function):
-        func_name, func_args = dict(raw_function).popitem()
-        return TEMPLATE_FUNCTIONS[func_name](func_args,
-                                             scope=scope,
-                                             context=context,
-                                             path=path,
-                                             raw=raw_function)
+    f = get_function(raw_function)
+    if f:
+        return_type = raw_function.get('type', None)
+        return f[0](f[1],
+                    scope=scope,
+                    context=context,
+                    path=path,
+                    raw=raw_function,
+                    return_type=return_type)
     return raw_function
 
 
@@ -1440,6 +1520,10 @@ class _EvaluationHandler(object):
     def evaluate_function(self, func):
         return func.evaluate(self)
 
+    @property
+    def runtime_only_evaluation(self):
+        return False
+
 
 class _PlanEvaluationHandler(_EvaluationHandler):
     def __init__(self, plan, runtime_only_evaluation):
@@ -1476,6 +1560,14 @@ class _PlanEvaluationHandler(_EvaluationHandler):
         raise exceptions.FunctionEvaluationError(
             "`get_sys` should be evaluated at runtime only")
 
+    def get_consumers(self, prop):
+        raise exceptions.FunctionEvaluationError(
+            "`get_consumers` should be evaluated at runtime only")
+
+    @property
+    def runtime_only_evaluation(self):
+        return self._runtime_only_evaluation
+
 
 class _RuntimeEvaluationHandler(_EvaluationHandler):
     def __init__(self, storage):
@@ -1556,6 +1648,13 @@ class _RuntimeEvaluationHandler(_EvaluationHandler):
     def get_sys(self, entity, prop):
         return self._storage.get_sys(entity, prop)
 
+    def get_consumers(self, prop):
+        return self._storage.get_consumers(prop)
+
+    @property
+    def runtime_only_evaluation(self):
+        return True
+
 
 def plan_evaluation_handler(plan, runtime_only_evaluation=False):
     return _PlanEvaluationHandler(plan, runtime_only_evaluation)
