@@ -45,13 +45,13 @@ class AbstractImportResolver(ABC):
     def resolve(self, import_url):
         raise NotImplementedError
 
-    def fetch_import(self, import_url):
+    def fetch_import(self, import_url, **kwargs):
         if is_remote_resource(import_url):
             return self.resolve(import_url)
         return read_import(import_url)
 
     @abc.abstractmethod
-    def retrieve_plugin(self, import_url):
+    def retrieve_plugin(self, import_url, **kwargs):
         raise NotImplementedError
 
 
